#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from flask import Blueprint, render_template, redirect, url_for, request, flash
from werkzeug.security import generate_password_hash, check_password_hash # https://werkzeug.palletsprojects.com/en/0.15.x/utils/
from flask_login import login_user, logout_user, login_required
from . import User, db
import pyotp, qrcode # https://pyotp.readthedocs.io/en/latest/#

# blueprint auth_page
auth = Blueprint('auth_page',__name__)

# objekt pre pracu s OTP pocitadlom
m_otp = pyotp.totp.TOTP('ABCD3FGH2JKL3NOP')
# pre docasne ulozenie emailu
usr_email = ''

# registracia
@auth.route('/register')
def register():
    # zobrazi stranku register.html
    return render_template('register.html')
# registracia - POST
@auth.route('/register',methods=['POST'])
def register_post():
    # email a heslo z formulara
    email = request.form['email']
    password = request.form['password']
    # ak je prazdy email alebo heslo
    if not email or not password:
        flash('Prosím skontroluj zadané údaje a skús znovu')
        return redirect(url_for('auth_page.register'))
    # vrati prvy riadok z dB na ktorom sa rovnaju emaily
    usr = User.query.filter_by(email = email).first()
    # ak email existuje
    if usr:
        flash('Zadaný email už existuje')
        return redirect(url_for('auth_page.register'))
    # from https://flask-sqlalchemy.palletsprojects.com/en/2.x/quickstart/
    # novy riadok v dB
    usr = User(email = email,password = generate_password_hash(password,method = 'sha256'))
    db.session.add(usr)
    db.session.commit()
    # docasne ulozenie emailu
    global usr_email
    usr_email = email
    # vykona funkciu otp()
    return redirect(url_for('auth_page.otp'))

# OTP
@auth.route('/otp')
def otp():
    # QR kod z OTP URI
    img = qrcode.make(m_otp.provisioning_uri(usr_email,issuer_name = 'IsAu1'))
    # ulozenie QR kodu do obrazka
    img.save('cars_v3/static/otp_'+usr_email+'.png')
    # zobrazi stranku otp.html
    return render_template('otp.html',email = usr_email)
# OTP - POST
@auth.route('/otp',methods=['POST'])
def otp_post():
    # po stlaceni tlacidla vykona funkciu login()
    return redirect(url_for('auth_page.login'))

# prihlasenie
@auth.route('/login')
def login():


    usr = User.query.filter_by(email = 'marek@marek.sk').first()
    login_user(usr)
    return redirect(url_for('inner_page.index'))


    # zobrazi stranku login.html
    return render_template('login.html')
# prihlasenie - POST
@auth.route('/login',methods = ['POST'])
def login_post():
    # email, heslo a kod z formulara
    email = request.form['email']
    password = request.form['password']
    otpcode = request.form['code']
    # ak je prazdy email alebo heslo alebo otp kod
    if not email or not password or not otpcode:
        flash('Prosím skontroluj zadané údaje a skús znovu')
        return redirect(url_for('auth_page.login'))
    # vrati prvy riadok z dB na ktorom sa rovnaju emaily
    usr = User.query.filter_by(email = email).first()
    # ak email neexistuje alebo sa nezdhoduju hesla
    if not usr or not check_password_hash(usr.password,password): 
        flash('Prosím skontroluj zadané údaje a skús znovu')
        return redirect(url_for('auth_page.login'))
    # na dopracovanie - rovnaky kod pre kazdeho?
    if (otpcode != m_otp.now()):
        flash('Zlý OTP kód')
        return redirect(url_for('auth_page.login'))
    # prihlasenie pouzivatela pomocou flask-login
    login_user(usr)
    # vstup do internetho webu - vykona funkciu inner_page.index()
    return redirect(url_for('inner_page.index'))

# odhlasenie
@auth.route('/logout')
@login_required # z flask-login - pouzivatel musi byt prihlaseny
def logout():
    # odhlasenie pouzivatela pomocou flask-login
    logout_user()
    # navrat do externeho webu - vykona funkciu outer_page.index()
    return redirect(url_for('outer_page.index'))
