from flask import Blueprint, render_template, redirect, url_for, request
from flask_login import login_required, current_user
from . import Car, db

import pygal
from pygal.style import CleanStyle

from bokeh.resources import INLINE
from bokeh.models import ColumnDataSource
from bokeh.models.widgets import TableColumn, DataTable
from bokeh.layouts import column, widgetbox
from bokeh.embed import components

import matplotlib.pyplot as plt
from matplotlib import colors
import numpy as np
from mpl_toolkits.mplot3d import Axes3D

from io import BytesIO
import base64

import pandas

# blueprint inner_page
inner = Blueprint('inner_page',__name__)

# List pre vybrane auta
sel_cars = []

# inside/index.html
@inner.route('/inside')
@login_required # z flask-login - pouzivatel musi byt prihlaseny
def index():
    # zobrazi stranku inside/index.html
    return render_template('inside/index.html',email = current_user.email)

# inside/new_car.html
@inner.route('/inside/new_car')
@login_required # z flask-login - pouzivatel musi byt prihlaseny
def new_car():
    # zobrazi stranku inside/new_car.html
    return render_template('inside/new_car.html',email = current_user.email)
# inside/new_car.html - POST
@inner.route('/inside/new_car',methods = ['POST'])
@login_required # z flask-login - pouzivatel musi byt prihlaseny
def new_car_post():
    # udaje z formulara
    manufacturer = request.form['manufacturer']
    model = request.form['model']
    fuel = request.form['fuel']
    power = request.form['power']
    torque = request.form['torque']
    drive = request.form['drive']
    gear = request.form['gear']
    fuelusage = request.form['fuelusage']
    height = request.form['height']
    width = request.form['width']
    length = request.form['length']
    wheelbase = request.form['wheelbase']
    groundclearance = request.form['groundclearance']
    cargovolume = request.form['cargovolume']
    trailerweight = request.form['trailerweight']
    price = request.form['price']
    # from https://flask-sqlalchemy.palletsprojects.com/en/2.x/quickstart/
    # novy riadok v dB
    car = Car(manufacturer = manufacturer,model = model,fuel = fuel,power = power,torque = torque,drive = drive,gear = gear,fuelusage = fuelusage,height = height,width = width,length = length,wheelbase = wheelbase,groundclearance = groundclearance,cargovolume = cargovolume,trailerweight = trailerweight,price = price)
    db.session.add(car)
    db.session.commit()
    # zobrazi stranku inside/new_car.html
    return redirect(url_for('inner_page.new_car'))

# inside/select_cars.html
@inner.route('/inside/select_cars')
@login_required # z flask-login - pouzivatel musi byt prihlaseny
def select_cars():
    # riadky z dB
    rows = Car.query.all()
    # zobrazi stranku inside/select_cars.html
    return render_template('inside/select_cars.html',email = current_user.email,rows = rows)
# inside/select_cars.html - POST
@inner.route('/inside/select_cars',methods = ['POST'])
@login_required # z flask-login - pouzivatel musi byt prihlaseny
def select_cars_post():
    # pracujeme s globalnou premennou
    global sel_cars
    # vymazanie Listu
    sel_cars.clear()
    # riadky z dB
    all_rows = Car.query.all()
    # pre vsetky zaznamy
    for row in all_rows:
        # stav checkboxu, ktoreho name = id_"id zaznamu"
        chbox = request.form.get("id_{}".format(row.id))
        # ak je checkbox zaskrtnuty
        if chbox:
            # pridanie zaznamu do Listu
            sel_cars.append(row)
    # ak nebolo vybrane ziadne auto
    if not sel_cars:
        return redirect(url_for('inner_page.select_cars'))
    # tlacidlo Porovnaj
    if request.form['bsubmit'] == "1":
        # zobrazi stranku inside/compare_cars.html
        return redirect(url_for('inner_page.compare_cars'))
    # tlacidlo Porovnaj detaily
    elif request.form['bsubmit'] == "2":
        # zobrazi stranku inside/cars_details.html
        return redirect(url_for('inner_page.cars_details'))

# inside/compare_cars.html
@inner.route('/inside/compare_cars')
@login_required # z flask-login - pouzivatel musi byt prihlaseny
def compare_cars():
    # pracujeme s globalnou premennou
    global sel_cars
    # pygal --------------------------------------------------------------
    chart1 = pygal.HorizontalBar(style = CleanStyle)
    chart1.force_uri_protocol = 'http'
    chart1.title = 'Porovnanie výkonu'
    chart1.x_title = 'Výkon'
    chart1.y_title = 'Výrobca'
    chart2 = pygal.XY(stroke = False)
    chart2.force_uri_protocol = 'http'
    chart2.title = 'Porovnanie ceny'
    chart2.x_title = 'Výrobca'
    chart2.y_title = 'Cena'
    # 16 prazdnych Listov
    d = [[] for i in range(16)]
    # pre chart2.x_labels
    xl = []
    # naplnenie Listov udajmi o autach
    num = 0
    for item in sel_cars:
        num += 1
        d[0].append(item.manufacturer)
        d[1].append(item.model)
        d[2].append(item.fuel)
        d[3].append(item.power)
        d[4].append(item.torque)
        d[5].append(item.drive)
        d[6].append(item.gear)
        d[7].append(item.fuelusage)
        d[8].append(item.height)
        d[9].append(item.width)
        d[10].append(item.length)
        d[11].append(item.wheelbase)
        d[12].append(item.groundclearance)
        d[13].append(item.cargovolume)
        d[14].append(item.trailerweight)
        d[15].append(item.price)
        chart1.add(item.manufacturer,item.power)
        chart2.add(item.manufacturer,[(num,item.price)],dots_size = 10)
        xl.append({'label':item.manufacturer,'value':num})
    # pygal grafy
    chart1 = chart1.render_data_uri()
    chart2.x_labels = xl
    chart2 = chart2.render_data_uri()
    # bokeh --------------------------------------------------------------
    # js a css
    bokeh_js = INLINE.render_js()
    bokeh_css = INLINE.render_css()   
    # udaje o autach ako Listy v jednotlivych stlpcoch tabulky
    data = ColumnDataSource({'d0':d[0],
                            'd1':d[1],
                            'd2':d[2],
                            'd3':d[3],
                            'd4':d[4],
                            'd5':d[5],
                            'd6':d[6],
                            'd7':d[7],
                            'd8':d[8],
                            'd9':d[9],
                            'd10':d[10],
                            'd11':d[11],
                            'd12':d[12],
                            'd13':d[13],
                            'd14':d[14],
                            'd15':d[15]})
    # stlpce tabulky so stanovenou sirkou
    columns = [TableColumn(field = 'd0',title = 'Výrobca',width = 80),
            TableColumn(field = 'd1',title = 'Značka',width = 80),
            TableColumn(field = 'd2',title = 'Palivo',width = 80),
            TableColumn(field = 'd3',title = 'Výkon',width = 80),
            TableColumn(field = 'd4',title = 'Krútiaci moment',width = 100),
            TableColumn(field = 'd5',title = 'Pohon',width = 80),
            TableColumn(field = 'd6',title = 'Prevodovka',width = 80),
            TableColumn(field = 'd7',title = 'Spotreba',width = 80),
            TableColumn(field = 'd8',title = 'Výška',width = 60),
            TableColumn(field = 'd9',title = 'Šírka',width = 60),
            TableColumn(field = 'd10',title = 'Dĺžka',width = 60),
            TableColumn(field = 'd11',title = 'Rázvor',width = 60),
            TableColumn(field = 'd12',title = 'Svetlá výška',width = 80),
            TableColumn(field = 'd13',title = 'Objem kufra so zdvihnutými sedadlami',width = 200),
            TableColumn(field = 'd14',title = 'Hmotnosť nebrzdeného prívesu',width = 200),
            TableColumn(field = 'd15',title = 'Cena',width = 60)]
    # bokeh tabulka - bez stlpca index, so stanovenou sirkou a vyskou
    data_table = DataTable(source = data,columns = columns,index_position = None,fit_columns = False,width = 1000,height = 200)
    # bokeh figure
    fig = column(widgetbox(data_table))
    # pre render_template
    plot_script,plot_div = components(fig)
    # matplotlib ---------------------------------------------------------
    fig = plt.figure(figsize=(8,6))
    ax = fig.add_subplot(111,projection = '3d')
    fig.subplots_adjust(right = 1.0,top = 1.0)
    ax.set_title('Porovnanie spotreby',pad = 15)
    ax.set_xlabel('Výrobca',labelpad = 15)
    ax.set_zlabel('Spotreba (l/100km)')
    # List standardnych farieb
    default_colors = [colors.to_rgba(c) for c in plt.rcParams['axes.prop_cycle'].by_key()['color']]
    # ak bolo vybrane aspon 1 auto
    if num != 0:
        # popis osi X
        x = np.arange(num)
        ax.set_xticks(x)
        ax.set_xticklabels(d[0])
        # popis osi Y
        y = np.zeros(num)
        ax.set_yticklabels([])
        # priprava farieb
        c = []
        for i in range(num):
            c.append(default_colors[i])
        # 3D graf
        ax.bar3d(x,y,y,0.25,0.1,d[7],color = c)
    # priprava obrazku
    img_bytes = BytesIO()
    plt.savefig(img_bytes,format = 'png')
    img_bytes.seek(0)
    img_fullenc = base64.b64encode(img_bytes.getvalue())
    img_enc = str(img_fullenc)[2:-1]
    
    # zobrazi stranku inside/compare_cars.html
    return render_template('inside/compare_cars.html',email = current_user.email,
                        bokeh_js = bokeh_js,bokeh_css = bokeh_css,plot_script = plot_script,plot_div = plot_div, # bokeh
                        img_enc = img_enc, # matplotlib
                        chart1 = chart1,chart2 = chart2) # pygal

# inside/cars_details.html
@inner.route('/inside/cars_details')
@login_required # z flask-login - pouzivatel musi byt prihlaseny
def cars_details():
     # pracujeme s globalnou premennou
    global sel_cars
    # nacitanie udajov zo suboru Excel
    df = pandas.read_excel('cars_v3/cars_details.xls',sheet_name = 'details')
    # nahradenie 'nan' prazdnym retazcom ''
    df = df.replace(np.nan,'',regex = True)
    # ak nebolo vybrane ziadne auto
    if not sel_cars:
        return redirect(url_for('inner_page.select_cars'))
    # vlastnosti auta z prisluchajuceho stlpca Excel tabulky
    prop = []
    # cena vlastnosti auta z prisluchajuceho stlpca Excel tabulky
    prop_prices = []
    # pocet vybranych aut
    sc = 0
    # pre vsetky vybrane auta
    for item in sel_cars:
        prop.append(df[item.manufacturer]) # vlastnosti auta ako List
        prop_prices.append(df[item.manufacturer + ' ceny']) # cena vlastnosti auta ako List
        sc += 1
        if sc == 1:
            # vlastnosti 1. vybraneho auta v poradi
            cars_properties = df[item.manufacturer].copy()

    # pocet riadkov tabulky s vlastnostami
    propnum = len(prop[0])
    # prazdny List pre kazdy riadok tabulky s vlastnostami
    data = [[] for i in range(propnum)]

    # transformacia udajov zo stlpcov do riadkov
    for pn in range(propnum): # pre vsetky riadky = vlastnosti auta
        for i in range(sc): # pre vsetky vybrane auta
            # ak je uvedena cena vlastnosti
            if prop_prices[i][pn] != '':
                # ak je v cene uvedene 'nie', tzn. vlastnost nie je u daneho auta dostupna alebo zistena
                if prop_prices[i][pn] == 'nie':
                    data[pn].append('0')
                # inak sa uvedie cena vlastnosti
                else:
                    data[pn].append('{:.0f}'.format(prop_prices[i][pn]))
            # ak nie je vlastnost uvedena
            elif prop[i][pn] == '':
                data[pn].append('0')
            # ak je vlastnost rovnaka ako vlastnost 1. vybraneho auta
            elif prop[i][pn] == cars_properties[pn]:
                data[pn].append('1')
            # konkretizovana vlastnost
            else:
                data[pn].append(prop[i][pn])

    # zobrazi stranku inside/cars_details.html
    return render_template('inside/cars_details.html',email = current_user.email,
                           scars = sel_cars,propnum = propnum,column1 = cars_properties,data = data)
